@@ -16,28 +16,18 @@ class Candidate(Your):
     Candidate Class
 
     Args:
-
-        fp : String or a list of files. It can either filterbank or psrfits files.
-
+        fp Union[str, List]: String or a list of files. It can either filterbank or psrfits files.
         dm (float): Dispersion Measure of the candidate
-
         tcand (float): start time of the candidate in seconds at the highest frequency channel
-
         width (int): pulse width of the candidate in samples
-
         label (int): 1 for pulsars/FRBs, 0 for RFI
-
         snr (float): Signal to Noise Ratio
-
         min_samp (int): Minimum number of time samples
-
         device (int): GPU ID if using GPUs
-
         kill_mask (numpy.ndarray): Boolean mask of channels to kill
     """
 
-    def __init__(self, fp=None, dm=None, tcand=0, width=0, label=-1, snr=0, min_samp=256, device=0,
-                 kill_mask=None):
+    def __init__(self, fp=None, dm=None, tcand=0, width=0, label=-1, snr=0, min_samp=256, device=0, kill_mask=None):
         Your.__init__(self, fp)
         self.dm = dm
         self.tcand = tcand
@@ -59,13 +49,10 @@ class Candidate(Your):
         Save the candidate to a hdf5 file
 
         Args:
-
             out_dir (str): path to the output directory
-
             fnout (str): output name of the file
 
         Returns:
-
             str: output name of the file
 
         """
@@ -116,14 +103,12 @@ class Candidate(Your):
 
     def dispersion_delay(self, dms=None):
         """
-        Caluclaute the dispersion delay for the candidate DM or at given dispersion DM
+        Calculate the dispersion delay for the candidate DM or at given dispersion DM
 
         Args:
-
             dms (Union[float,np.ndarray]) : DM or a list of DMs
 
         Returns:
-
             Union[float, np.ndarray]: dispersion delay in seconds
 
         """
@@ -137,11 +122,8 @@ class Candidate(Your):
         Get a chunk of data. The data is saved in `self.data`.
 
         Args:
-
             tstart (float): start time of the chunk in seconds
-
             tstop (float): stop time of the chunk in seconds
-
             for_preprocessing (bool): if the data is to be preprocessed later. This will modify the number of samples
             read based on the width of the candidate
 
@@ -213,13 +195,10 @@ class Candidate(Your):
         Dedisperse a chunk of data. Saves the dedispersed chunk in `self.dedispersed`.
 
         Note:
-
             Our method rolls the data around while dedispersing it.
 
         Args:
-
             dms (float): The DM to dedisperse the data at.
-
             target (str): 'CPU' to run the code on the CPU or 'GPU' to run it on a GPU.
 
         """
@@ -248,11 +227,9 @@ class Candidate(Your):
         Create a dedispersed time series
 
         Note:
-
             Our method rolls the data around while dedispersing it.
 
         Args:
-
             dms (float): The DM to dedisperse the data at.
 
         Returns:
@@ -276,13 +253,10 @@ class Candidate(Your):
         Generates DM-time array of the candidate by dedispersing at adjacent DM values. Saves the data in `self.dmt`.
 
         Note:
-
             Our method rolls the data around while dedispersing it.
 
         Args:
-
             dmsteps (int) : Number of DMs to dedisperse at.
-
             target (str): 'CPU' to run the code on the CPU or 'GPU' to run it on a GPU.
 
         """
@@ -301,11 +275,9 @@ class Candidate(Your):
         Calculates the SNR of the candidate
 
         Args:
-
-            time_series: time series array to calculate the SNR of
+            time_series (np.ndarray) : time series array to calculate the SNR of
 
         Returns:
-
             float: SNR
         """
         if time_series is None and self.dedispersed is None:
@@ -326,11 +298,9 @@ class Candidate(Your):
         Calculate more precise value of the DM by interpolating between DM values to maximise the SNR
 
         Note:
-
             This function has not been fully tested.
 
         Returns:
-
             optimnised DM, optimised SNR
         """
         if self.data is None:
@@ -353,19 +323,13 @@ class Candidate(Your):
         Decimate FT or DMT data.
 
         Todo:
-
             * Update candidate parameters as per decimation factor
 
         Args:
-
             key (str): Keywords to chose which data to decimate ('dmt' or 'ft')
-
             decimate_factor (int): Number of samples to average
-
             axis (int): Axis to decimate along
-
             pad (bool): Optional argument if padding is to be done
-
             **kwargs: kwargs for numpy.pad
         """
         if key == 'dmt':
@@ -387,17 +351,12 @@ class Candidate(Your):
         Resize FT or DMT data
 
         Todo:
-
             * Update candidate parameters as per final size
 
         Args:
-
             key (str): Keywords to chose which data to resize ('dmt' or 'ft')
-
             size: Final size of the data array required
-
             axis (int) : Axis to resize alone
-
             **kwargs: Arguments for skimage.transform resize function
 
         """
