@@ -31,12 +31,10 @@ def unpack_2bit(data):
     Unpack 2-bit data that has been read in as bytes.
 
     Args:
-
-        data: array of unsigned 2-bit ints packed into an array of bytes.
+        data (np.ndarray) : array of unsigned 2-bit ints packed into an array of bytes.
 
     Returns:
-
-        unpacked array. The size of this array will be four times the size of the input data.
+        np.ndarray: unpacked array. The size of this array will be four times the size of the input data.
 
     """
     piece0 = np.bitwise_and(data >> 0x06, 0x03)
@@ -51,12 +49,10 @@ def unpack_4bit(data):
     Unpack 4-bit data that has been read in as bytes.
 
     Args:
-
-        data: array of unsigned 4-bit ints packed into an array of bytes.
+        data (np.ndarray) : array of unsigned 4-bit ints packed into an array of bytes.
 
     Returns:
-
-        unpacked array. The size of this array will be twice the size of the input data.
+        np.ndarray: unpacked array. The size of this array will be twice the size of the input data.
 
     """
     piece0 = np.bitwise_and(data >> 0x04, 0x0F)
@@ -72,55 +68,31 @@ class PsrfitsFile(object):
     [psrfits](https://github.com/scottransom/presto/blob/master/python/presto/psrfits.py ).
 
     Args:
-
         psrfitslist (str): list of files
 
     Attributes:
-
         filename (str): Name of the first file
-
         filelist (list): List of files
-        
         fileid (int): Index of the current file
-        
         fits (obj): fits object of the current file read
-        
         specinfo (obj): Object of class SpectraInfo for the given file list
-        
         header (list): Header of the fits file
-        
         source_name (str): Source Name
-
         machine_id (int) : Machine ID
-
         barycentric (int): If 1 the data is barycentered
-
         pulsarcentric (int): Is the data in pulsar's frame of reference?
-
         src_raj (float): RA of the source (HHMMSS.SS)
-
         src_deg (float): Dec of the source (DDMMSS.SS)
-
         az_start (float): Telescope Azimuth (degrees)
-
         za_start (float): Telescope Zenith Angle (degrees)
-
         fch1 (float): Frequency of first channel (MHz))
-
         foff (float): Channel bandwidth (MHz)
-
         nchans (int): Number of channels
-
         nbeams (int): Number of beams in the rcvr.
-
         ibeam (int): Beam number
-
         nbits (int): Number of bits the data are recorded in.
-
         tstart (float): Start MJD of the data
-
         tsamp (float): Sampling interval (seconds)
-
         nifs (int): Number of IFs in the data.
         
     """
@@ -167,8 +139,7 @@ class PsrfitsFile(object):
         """
 
         Returns:
-
-            Total number of spectra in all files in filelist
+            int : Total number of spectra in all files in filelist
 
         """
         return int(self.specinfo.spectra_per_subint * np.sum(self.specinfo.num_subint))
@@ -178,8 +149,7 @@ class PsrfitsFile(object):
         Native number of total spectra in all the files. This will be made a property so that it can't be overwritten
 
         Returns:
-
-            Total number of spectra in all files in filelist
+            int : Total number of spectra in all files in filelist
 
         """
         return int(self.specinfo.spectra_per_subint * np.sum(self.specinfo.num_subint))
@@ -189,8 +159,7 @@ class PsrfitsFile(object):
         This will be made a property so that it can't be overwritten.
 
         Returns:
-
-            Native sampling time of the file.
+            float : Native sampling time of the file.
 
         """
         return self.specinfo.dt
@@ -200,8 +169,7 @@ class PsrfitsFile(object):
         This will be made a property so that it can't be overwritten.
 
         Returns:
-
-             Native channel bandwidth
+             float : Native channel bandwidth
 
         """
         return self.bw / self.nchan
@@ -211,8 +179,7 @@ class PsrfitsFile(object):
         This will be made a property so that it can't be overwritten.
 
         Returns:
-
-            Native number of channels in the filterbank
+            int: Native number of channels in the file
 
         """
         return self.nchan
@@ -224,18 +191,13 @@ class PsrfitsFile(object):
         Applys scales, weights, and offsets to the data.
 
         Args:
-
-            isub: index of subint (first subint is 0)
-
-            apply_weights: If True, apply weights. (Default: apply weights)
-
-            apply_scales: If True, apply scales. (Default: apply scales)
-
-            apply_offsets: If True, apply offsets. (Default: apply offsets)
+            isub (int) : index of subint (first subint is 0)
+            apply_weights (bool) : If True, apply weights. (Default: apply weights)
+            apply_scales (bool) : If True, apply scales. (Default: apply scales)
+            apply_offsets (bool) : If True, apply offsets. (Default: apply offsets)
 
         Returns:
-
-            Subint data with scales, weights, and offsets applied in float32 dtype with shape (nsamps,nchan).
+            np.ndarray : Subint data with scales, weights, and offsets applied in float32 dtype with shape (nsamps,nchan).
 
         """
         sdata = self.fits['SUBINT'].data[isub]['DATA']
@@ -301,12 +263,10 @@ class PsrfitsFile(object):
         Return weights for a particular subint.
 
         Args:
-
-           isub: index of subint (first subint is 0)
+           isub (int) : index of subint (first subint is 0)
             
         Returns:
-
-            weights: Subint weights. (There is one value for each channel)
+            np.ndarray : Subint weights. (There is one value for each channel)
 
         """
         return self.fits['SUBINT'].data[isub]['DAT_WTS']
@@ -316,12 +276,10 @@ class PsrfitsFile(object):
         Return scales for a particular subint.
 
         Args:
-
-             isub: index of subint (first subint is 0)
+             isub (int) : index of subint (first subint is 0)
             
         Returns:
-
-            scales: Subint scales. (There is one value for each channel)
+            np.ndarray : Subint scales. (There is one value for each channel)
 
         """
         return self.fits['SUBINT'].data[isub]['DAT_SCL']
@@ -331,12 +289,10 @@ class PsrfitsFile(object):
         Return offsets for a particular subint.
 
         Args:
-
-            isub: index of subint (first subint is 0)
+            isub (int) : index of subint (first subint is 0)
             
         Returns:
-
-            offsets: Subint offsets. (There is one value for each channel)
+            np.ndarray : Subint offsets. (There is one value for each channel)
 
         """
         return self.fits['SUBINT'].data[isub]['DAT_OFFS']
@@ -346,14 +302,12 @@ class PsrfitsFile(object):
         Return 2D array of data from PSRFITS files.
  
         Args:
-
-            nstart: Starting sample
-
-            nsamp: number of samples to read
+            nstart (int) : Starting sample
+            nsamp (int) : number of samples to read
+            pol (int) : which polarization to return
  
         Returns:
-
-            data: Time-Frequency numpy array
+            np.ndarray : Time-Frequency numpy array
 
         """
         # Calculate starting subint and ending subint
@@ -452,9 +406,7 @@ class SpectraInfo:
     Class to read the header of fits files
 
     Args:
-
-        filenames: list of fits files
-
+        filenames (list) : list of fits files
     """
 
     def __init__(self, filenames):
@@ -803,8 +755,7 @@ def DATEOBS_to_MJD(dateobs):
     Convert DATE-OBS string from PSRFITS primary HDU to a MJD.
 
     Returns:
-
-         a 2-tuple: (integer part of MJD, fractional part of MJD)
+         tuple: (integer part of MJD, fractional part of MJD)
 
     """
     # Parse string using regular expression defined at top of file
@@ -818,7 +769,8 @@ def DATEOBS_to_MJD(dateobs):
 
 def is_PSRFITS(filename):
     """
-    Return True if filename appears to be PSRFITS format. Return False otherwise.
+    Return:
+        bool: True if filename appears to be PSRFITS format. Return False otherwise.
 
     """
     with pyfits.open(filename, mode='readonly', memmap=True) as hdus:
